/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(__LABEL_H)
#define __LABEL_H

class xFont;
class Texture;
class GhostLabel;

typedef vector<GhostLabel *>GhostLabelList;
typedef vector<GhostLabel *>::iterator GhostLabelListIterator;

class Label
{
public:

	Label(const string &name);
	virtual ~Label();

	void load(HINSTANCE hInstance, HWND box = 0, bool dropShadow = false);
	void reconfigure(bool refresh = false);

	Texture *getBackground() const { return background; };
	xFont *getFont() const { return font; };

	bool isAlwaysOnTop() const { return alwaysOnTop; };
	bool isVisible() const { return visible; };
	bool isTextboxVisible() const { return textboxVisible; };
	bool isStartHidden() const { return startHidden; }
	bool isAlphaMapSet() const { return alphaMap; };

	int getX() const { return actualx; };
	int getY() const { return actualy; };
	int getHeight() const { return actualheight; };
	int getWidth() const { return actualwidth; };

	int getTransparencyMode() const { return transparencyMode; };

	int getScrolling() const { return scroll; };

	const HWND &getBox() const { return box; };
	const HWND &gethWnd() const { return hWnd; };
	const HWND &getTexteditBoxhWnd() const { return hWndTextBox; };
	const string &getName() const { return name; };
	const string &getText() const { return text; };

	HCURSOR getTextCursor() { return textcursor; };
	string getTextCursorPath() { return textcursorPath; };

	void setBox(HWND newparent);
	void setBackground(Texture *newbackground);
	void setFont(xFont *newfont);
	void setText(const string &text, bool updateBackup = false, bool textEditBox = false);

	void setAlwaysOnTop(bool alwaysOnTop);

	void setScrolling(int scrolling);
	void setScrollLimit(int limit);
	
	void setAlpha(int newAlphaTransparency = 255);
	void setAlphaFade(int newAlphaTransparency, int step, int delay);

	void setAnimation(bool start, int loops = -1, int startFrame = 1);

	bool checkEnterLeaveRegion(int x, int y, StringList inputlist);
	bool checkRegion(int x, int y, StringList inputlist, bool internal = false);
	
	void repaint(bool invalidateCache = false);
	void updateRegion();

	void reposition(int type, int newx, int newy, int newwidth, int newheight, int steps = 0, int time = 0, bool repositionBy = false, bool movecommand = true, Label* sender = NULL);

	void hide(bool execute = false, bool textEditBox = false);
	void show(bool execute = false, bool textEditBox = false);
	void showHide(double timeout, bool execute = false);

	void previous();
	void next();
	void update();

	void clipboardCopy(string prefix);
	void clipboardPaste(string prefix);

	bool bUseFahrenheit;

	//Actions (with Public Access)
	//--------------------------------------------
	string resizeCommand;
	string focusCommand;
	string unfocusCommand;

	//Label Group
	string labelGroup;

	//AutoSize
	int autoWidthMode;
	int autoHeightMode;

	//Internal Used Settings (with Public Access)
	//--------------------------------------------
	//ModuleHook
	bool hasHookedModules;
	StringList moduleHook;
	
	//Scroll AddOns
	bool frozen;
	int scrollBackup;
	
	//Docking
	StringList dockedToList;
	LabelList hasDockedLabelList;
	
	//Type: 0 left, 1 top, 2 right, 3 bottom, topleft 4, topright 5, bottomleft 6, bottomright 7
	int dockedtoSide;

	bool FindDockedLabel(Label* findLabel);
	
	//TextEditBox
	WNDPROC wpOrigEditProc;

private:
	
	//BehindWindow
	string afterClass;
	string afterTitle;

	GhostLabelList ghostlabelList;

	HCURSOR cursor;
	HCURSOR textcursor;
	HCURSOR handcursor;
	string cursorPath;
	string textcursorPath;

	HWND hWnd;
	HINSTANCE hInstance;
	HWND box;

	HWND hWndTextBox;

	string enterAction;

	int textboxPaintingMode;
	COLORREF textboxPlainColor;

	HFONT textboxhFont;
	string textboxXOrg;
	string textboxYOrg;
	string textboxWidthOrg;
	int textboxHeight;
	int tebleftBorder;
	int tebrightBorder;
	int tebtopBorder;
	int tebbottomBorder;
	bool textboxVisible;

	RECT TextEditBoxSizeBorder;
	RECT TextEditBoxSize;

	void enterActionProcessing(string input);

	string successCommand;
	string failCommand;

	HWND TooltipHints;

	HDC backgroundDC;
	HBITMAP backgroundBitmap;

	//Texture 
	Texture *background;
	Texture *texteditboxbackground;
	bool bUsingDefSkin;

	//Font
	xFont *font;

	bool alwaysOnTop;
	bool startHidden;

	int actualx;
	int actualy;
	int actualwidth;
	int actualheight;
	
	int borderMethod;
	int hoverborderMethod;
	int pressedborderMethod;

	int scrollPadLength;
	int scrollInterval;
	int scrollSpeed;
	int scroll;
	int scrollPosition;
	UINT scrollLimit;

	int scrollPause;
	int scrollActualStep;
	int scrollSteps;
	int scrollStepsInitial;

	//AutoSize
	int autoMinWidth;
	int autoMinHeight;
	int autoMaxWidth;
	int autoMaxHeight;
	int startCenterX;
	int startCenterY;

	int oldWidth;
	int oldHeight;

	int transparencyMode;
	bool transparencyNeeded;
	bool trackingIgnoreTrueTransparency;

	int alphaTransparency;
	bool alphaMap;
	bool fakealphaMap;
	int alphaFade;
	int alphaFadeBangStep;
	int alphaFadeBangEndValue;

	int updateInterval;
	bool alwaysUpdateContent;

	int align;
	int vertAlign;
	int leftBorder;
	int topBorder;
	int rightBorder;
	int bottomBorder;

	bool useTextFade;
	int textFadeLeft;
	int textFadeRight;
	int textFadeTop;
	int textFadeBottom;

	bool dynamicText;
	vector<string> originalText;
	string originalTextBackup;
	int current;
	int currentBackup;
	bool noTextSplit;
	string text;
	vector<string> hrefconfig;
	bool noEscape;

	bool dynamicTooltip;
	string originalTooltip;
	char tooltip[MAX_PATH_LENGTH];

	bool focusOnEnter;
	bool skipButton;

	bool autoHideIsRunning;

	//Actions
	string leftClickCommand;
	string leftClickDownCommand;
	string leftClickUpCommand;
	string leftDoubleClickCommand;
	string middleClickCommand;
	string middleClickDownCommand;
	string middleClickUpCommand;
	string middleDoubleClickCommand;
	string rightClickCommand;
	string rightClickDownCommand;
	string rightClickUpCommand;
	string rightDoubleClickCommand;
	string wheelDownCommand;
	string wheelUpCommand;
	string enterCommand;
	string leaveCommand;
	string dropCommand;
	string textChangeCommand;
	string moveCommand;
	string hideCommand;
	string showCommand;
	string hoverCommand;

	int hoverTime;

	bool fireOnMoveDuringMove;
	bool fireOnResizeDuringResize;

	string snapTo;
	int snapDistance;
	int dockOnSnapTo;
	StringList snapToList;

	bool moveable;
	int moveKey;
	int moveButton;

	bool useresizeBorder;
	int resizeBorderLeft;
	int resizeBorderRight;
	int resizeBorderTop;
	int resizeBorderBottom;

	StringList labelLeftClickRegions;
	StringList labelRightClickRegions;
	StringList labelMiddleClickRegions;

	//Animations
	int frameCounter;
	int loopCounter;

	int imageLoop;
	int imageFrameDelay;
	int imageFrameCount;
	int hoverimageLoop;
	int hoverimageFrameDelay;
	int hoverimageFrameCount;

	int alphaFadeStep;
	int alphaFadeDelay;

	//Internal Used Settings
	//------------------------
	//System
	string name;
	bool labelInitialized;

	bool visible;

	//Alpha Features
	bool layeredFlag;
	int UpdateAlpha;

	//Moving Features
	POINT savedpt;
	bool hasMoved;
	bool movemodifierkeyPressed;
	int moveButtonPressed;

	//Text Features
	StringList labelHREFClickRegions;
	bool mouseInsideHREF;

	//Mouse Features
	bool mousePressed;
	bool mouseInside;
	bool mouseInside2;

	void mouseTrack();

	//Image (Hover/Pressed/Animation) Features
	bool hoverActive;
	bool pressedActive;

	bool useHover;
	bool usePressed;

	bool normalAnim;
	bool hoverAnim;

	//Docking
	POINT oldPositionBackup;
	POINT oldSizeBackup;

	Label *sender;

protected:

	HBRUSH hTextBGBrush;
	HBRUSH GetBGBrush();

	void ontemplateButtonDown(int actualMouseClick, const string &clickDownCommand);
	void ontemplateButtonUp(const int &x, const int &y, int actualMouseClick, const StringList &labelClickRegions, const string &clickCommand, const string &clickUpCommand);

	void autoSize(bool doIt = false);
	
	int isCloseTo(POINT &pos, const POINT size, int x, int y, const int cx, const int cy, const int span);

	//Tooltips
	void addHint(HWND hWnd, LPSTR caption);
	void updateHint(HWND hWnd, LPSTR caption, bool updatesize = false);
	void removeHint(HWND hWnd);

	//Alpha (Normal and AlphaMap)
	void UpdateLabelAlpha(int newAlphaTransparency);
	
	void TextFadeProcessing(HDC workingDC, int width, int height);

	void relayMouseMessageToBox(UINT message, WPARAM wParam, LPARAM lParam);

	void onTextPaint(HDC desthdc, int width, int height);

	virtual void onLButtonDblClk(int x, int y);
	virtual void onLButtonDown(int x, int y);
	virtual void onLButtonUp(int x, int y);
	virtual void onMButtonDblClk(int x, int y);
	virtual void onMButtonDown(int x, int y);
	virtual void onMButtonUp(int x, int y);
	virtual void onRButtonDblClk(int x, int y);
	virtual void onRButtonDown(int x, int y);
	virtual void onRButtonUp(int x, int y);
	virtual void onWheelDown(int x, int y);
	virtual void onWheelUp(int x, int y);
	virtual void onMouseEnter();
	virtual void onMouseLeave();
	virtual void onMouseMove(int x, int y);
	virtual void onPaint(HDC hDC, RECT *specialRect = NULL);
	virtual void onWindowPosChanging(WINDOWPOS *windowPos);
	virtual void onMove(int x, int y);
	virtual void onSize(int width, int height);
	virtual void onTimer(int timerID);

	virtual LRESULT NcHitTest(int x, int y);

	virtual bool onWindowMessage(UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult);

public:

	static LRESULT WINAPI windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

};

#endif
